@@ -1,7 +1,12 @@
+from __future__ import annotations
+
 from typing import TYPE_CHECKING
 
 from starlite import ASGIConnection, BaseRouteHandler, NotAuthorizedException
 
+__all__ = ["roles_accepted", "roles_required"]
+
+
 if TYPE_CHECKING:
     from starlite.types import Guard
 
